/**
 * WildCAT: A Generic Framework for Context-Aware Applications.
 * Copyright (C) 2008 Bull S.A.S.
 * Copyright (C) 2008 EMN
 * Contact: wildcat@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * Authors:
 *		- Pierre-Charles David (initial)
 *		- Nicolas Loriant
 * --------------------------------------------------------------------------
 * $Id: JMSRemoteContext.java 350 2008-11-10 09:41:14Z loris $
 * --------------------------------------------------------------------------
 */
package org.ow2.wildcat.remote.dispatcher.jms;

import java.rmi.RemoteException;
import java.util.Set;
import java.util.concurrent.TimeUnit;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.ow2.wildcat.hierarchy.attribute.Attribute;
import org.ow2.wildcat.hierarchy.resource.Resource;
import org.ow2.wildcat.remote.RemoteContext;
import org.ow2.wildcat.remote.dispatcher.DispatcherException;
import org.ow2.wildcat.util.Messages;

/**
 * Frontend to access a remote Context through JMS.
 */
public class JMSRemoteContext implements RemoteContext {

    /**
     * logger.
     */
    private static Log logger = LogFactory.getLog(JMSRemoteContext.class);

    /**
     * Dispatcher to perform invocations on the associated remote context.
     */
    private final JMSDispatcher dispatcher;

    /**
     * Context name of the remote context.
     */
    private final String alias;

    public JMSRemoteContext(final JMSDispatcher dispatcher, final String alias) {
        this.dispatcher = dispatcher;
        this.alias = alias;
    }

    public void attachAttribute(final String attribute, final Attribute attr) {
        logger.debug(Messages.message(JMSRemoteContext.class.getName() + ".onsr"));
    }

    public void attachResource(final String resource, final Resource res) {
        logger.debug(Messages.message(JMSRemoteContext.class.getName() + ".onsr"));
    }

    public void createAttribute(final String attribute, final Object value) {
        ContextOperation co = new ContextOperation(this.alias,
                OperationType.CREATE_ATTRIBUTE, attribute, value);
        try {
            dispatcher.doRemoteContextOperation(co);
        } catch (DispatcherException e) {
            logger.warn("Unable to create the attribute with name: " + attribute, e);
        }
    }

    /**
     * @param attribute
     * @return
     */
    public Attribute getAttribute(final String attribute) {
        logger.debug(Messages.message(JMSRemoteContext.class.getName() + ".onsr"));
        return null;
    }

    public void createPeriodicAttributePoller(final String attribute, final long duration,
            final TimeUnit unit) {
        logger.debug(Messages.message(JMSRemoteContext.class.getName() + ".onsr"));
    }

    public void createResource(final String resource) {
        ContextOperation co = new ContextOperation(this.alias,
                OperationType.CREATE_RESOURCE, resource);
        try {
            dispatcher.doRemoteContextOperation(co);
        } catch (DispatcherException e) {
            logger.warn("Unable to create the resource with name: " + resource, e);
        }
    }

    public void createSymlink(final String link, final String target) {
        ContextOperation co = new ContextOperation(this.alias,
                OperationType.CREATE_SYMLINK, link, target);
        try {
            dispatcher.doRemoteContextOperation(co);
        } catch (DispatcherException e) {
            logger.warn("Unable to create the link with name: " + link, e);
        }
    }

    public String getName() {
        return this.alias;
    }

    public Object getValue(final String attribute) {
        ContextOperation co = new ContextOperation(this.alias,
                OperationType.GET_VALUE, attribute);
        ContextOperationResult cor;
        try {
            cor = this.dispatcher.doRemoteContextOperation(co);
        } catch (DispatcherException e) {
            logger.warn("Unable to get the value for attribute with name: " + attribute, e);
            return null;
        }
        return cor.getResult();
    }

    @SuppressWarnings("unchecked")
    public Set<String> list(final String resource) {
        ContextOperation co = new ContextOperation(this.alias,
                OperationType.LIST, resource);
        ContextOperationResult cor;
        try {
            cor = dispatcher.doRemoteContextOperation(co);
        } catch (DispatcherException e) {
            logger.warn("Unable to get the set of attributes for resource with name: " + resource, e);
            return null;
        }
        return (Set<String>) cor.getResult();
    }

    public Object setValue(final String attribute, final Object value) {
        ContextOperation co = new ContextOperation(this.alias,
                OperationType.SET_VALUE, attribute, value);
        ContextOperationResult cor;
        try {
            cor = dispatcher.doRemoteContextOperation(co);
        } catch (DispatcherException e) {
            logger.warn("Unable to set the value of attribute with name: " + attribute, e);
            return null;
        }
        return cor.getResult();
    }

    public void export() {
        logger.debug(Messages.message(JMSRemoteContext.class.getName() + ".onsr"));
    }

    /**
     * @param attribute
     */
    public void detachAttribute(final String attribute) {
        logger.debug(Messages.message(JMSRemoteContext.class.getName() + ".onsr"));
    }

    /**
     * @param resource
     */
    public void detachResource(final String resource) {
        logger.debug(Messages.message(JMSRemoteContext.class.getName() + ".onsr"));
    }

    /**
     * @param attribute
     */
    public void removePeriodicAttributePoller(final String attribute) {
        logger.debug(Messages.message(JMSRemoteContext.class.getName() + ".onsr"));
    }

    /**
     * @param resource
     * @return
     */
    public Resource getResource(final String resource) throws RemoteException {
        logger.debug(Messages.message(JMSRemoteContext.class.getName() + ".onsr"));
        return null;
    }

}
